/**
 * @file
 * VuoList_VuoDispersion C type definition.
 *
 * @copyright Copyright © 2012–2015 Kosada Incorporated.
 * This code may be modified and distributed under the terms of the MIT License.
 * For more information, see http://vuo.org/license.
 */

#ifndef VuoList_VuoDispersion_H
#define VuoList_VuoDispersion_H

/**
 * @ingroup VuoTypes
 * @defgroup VuoList_VuoDispersion VuoList_VuoDispersion
 * A list of @ref VuoDispersion elements.
 *
 * @{
 */

// This header is generated by vuo/type/list/generateVariants.sh.

#if defined(DOXYGEN) || !defined(VuoList_VuoDispersion_TYPE_DEFINED)
/// @{
/**
 * A list of @ref VuoDispersion elements.
 */
typedef const struct VuoList_VuoDispersion_struct { void *l; } * VuoList_VuoDispersion;
/// @}
#endif

/**
 * Creates a new list of @ref VuoDispersion elements.
 */
VuoList_VuoDispersion VuoListCreate_VuoDispersion(void);

/**
 * Makes a shallow copy of `list` — its items are retained (not copied) by the new list.
 */
VuoList_VuoDispersion VuoListCopy_VuoDispersion(const VuoList_VuoDispersion list);

/**
 * Returns the @ref VuoDispersion at @c index.
 * Index values start at 1.
 * If the list has no items, returns a default value.
 * Attempting to access an out-of-bounds index returns the first item in the list (if the index is 0), or last item in the list (if the index is greater than the list size).
 */
VuoDispersion VuoListGetValue_VuoDispersion(const VuoList_VuoDispersion list, const unsigned long index);

/**
 * Changes the @ref VuoDispersion at @c index.
 * Index values start at 1.
 * If the list has no items, nothing is changed.
 * Attempting to change an out-of-bounds index changes the first item in the list (if the index is 0), or last item in the list (if the index is greater than the list size).
 */
void VuoListSetValue_VuoDispersion(const VuoList_VuoDispersion list, const VuoDispersion value, const unsigned long index);

/**
 * Inserts the `VuoDispersion` immediately before `index`.
 * Index values start at 1.
 * Inserting at index 0 prepends the value to the list.
 * Inserting at an index beyond the last value in the list appends the value to the list.
 */
void VuoListInsertValue_VuoDispersion(const VuoList_VuoDispersion list, const VuoDispersion value, const unsigned long index);

/**
 * Prepends @c value to @c list.
 */
void VuoListPrependValue_VuoDispersion(VuoList_VuoDispersion list, const VuoDispersion value);

/**
 * Appends @c value to @c list.
 */
void VuoListAppendValue_VuoDispersion(VuoList_VuoDispersion list, const VuoDispersion value);

/**
 * Swaps the value at `indexA` with the value at `indexB`.
 */
void VuoListExchangeValues_VuoDispersion(VuoList_VuoDispersion list, const unsigned long indexA, const unsigned long indexB);

/**
 * Generates a random permutation of `list`.
 *
 * `chaos` ranges from 0 to 1.  When `chaos` is 1, a full Fisher–Yates shuffle is performed.  When less than 1, fewer iterations are performed.
 */
void VuoListShuffle_VuoDispersion(VuoList_VuoDispersion list, const double chaos);

/**
 * Reverses the order of the items in `list`.
 */
void VuoListReverse_VuoDispersion(VuoList_VuoDispersion list);

/**
 * Removes items from the list except those in the range specified by `startIndex` and `itemCount`.
 */
void VuoListCut_VuoDispersion(VuoList_VuoDispersion list, const signed long startIndex, const unsigned long itemCount);

/**
 * Removes the first value from @c list.
 */
void VuoListRemoveFirstValue_VuoDispersion(VuoList_VuoDispersion list);

/**
 * Removes the last value from @c list.
 */
void VuoListRemoveLastValue_VuoDispersion(VuoList_VuoDispersion list);

/**
 * Removes all values from @c list.
 */
void VuoListRemoveAll_VuoDispersion(VuoList_VuoDispersion list);

/**
 * Removes the `VuoDispersion` at `index`.
 * Index values start at 1.
 * Attempting to remove index 0 or an index beyond the last value in the list has no effect.
 */
void VuoListRemoveValue_VuoDispersion(VuoList_VuoDispersion list, const unsigned long index);

/**
 * Returns the number of elements in @c list.
 */
unsigned long VuoListGetCount_VuoDispersion(const VuoList_VuoDispersion list);

/**
 * Decodes the JSON object @c js to create a new value.
 *
 * @eg{
 *	["uno", "dos", "tres", "catorce"]
 * }
 */
VuoList_VuoDispersion VuoList_VuoDispersion_makeFromJson(struct json_object * js);

/**
 * Encodes @c value as a JSON object.
 */
struct json_object * VuoList_VuoDispersion_getJson(const VuoList_VuoDispersion value);

/**
 * Produces a brief human-readable summary of @c value.
 */
char * VuoList_VuoDispersion_getSummary(const VuoList_VuoDispersion value);

/// @{
/**
 * Automatically generated function.
 */
VuoList_VuoDispersion VuoList_VuoDispersion_makeFromString(const char *str);
char * VuoList_VuoDispersion_getString(const VuoList_VuoDispersion value);
/// @}

/**
 * @}
 */

#endif
