/**
 * @file
 * VuoList_VuoImageFormat C type definition.
 *
 * @copyright Copyright © 2012–2015 Kosada Incorporated.
 * This code may be modified and distributed under the terms of the MIT License.
 * For more information, see http://vuo.org/license.
 */

#ifndef VuoList_VuoImageFormat_H
#define VuoList_VuoImageFormat_H

/**
 * @ingroup VuoTypes
 * @defgroup VuoList_VuoImageFormat VuoList_VuoImageFormat
 * A list of @ref VuoImageFormat elements.
 *
 * @{
 */

// This header is generated by vuo/type/list/generateVariants.sh.

#if defined(DOXYGEN) || !defined(VuoList_VuoImageFormat_TYPE_DEFINED)
/// @{
/**
 * A list of @ref VuoImageFormat elements.
 */
typedef const struct VuoList_VuoImageFormat_struct { void *l; } * VuoList_VuoImageFormat;
/// @}
#endif

/**
 * Creates a new list of @ref VuoImageFormat elements.
 */
VuoList_VuoImageFormat VuoListCreate_VuoImageFormat(void);

/**
 * Makes a shallow copy of `list` — its items are retained (not copied) by the new list.
 */
VuoList_VuoImageFormat VuoListCopy_VuoImageFormat(const VuoList_VuoImageFormat list);

/**
 * Returns the @ref VuoImageFormat at @c index.
 * Index values start at 1.
 * If the list has no items, returns a default value.
 * Attempting to access an out-of-bounds index returns the first item in the list (if the index is 0), or last item in the list (if the index is greater than the list size).
 */
VuoImageFormat VuoListGetValue_VuoImageFormat(const VuoList_VuoImageFormat list, const unsigned long index);

/**
 * Changes the @ref VuoImageFormat at @c index.
 * Index values start at 1.
 * If the list has no items, nothing is changed.
 * Attempting to change an out-of-bounds index changes the first item in the list (if the index is 0), or last item in the list (if the index is greater than the list size).
 */
void VuoListSetValue_VuoImageFormat(const VuoList_VuoImageFormat list, const VuoImageFormat value, const unsigned long index);

/**
 * Inserts the `VuoImageFormat` immediately before `index`.
 * Index values start at 1.
 * Inserting at index 0 prepends the value to the list.
 * Inserting at an index beyond the last value in the list appends the value to the list.
 */
void VuoListInsertValue_VuoImageFormat(const VuoList_VuoImageFormat list, const VuoImageFormat value, const unsigned long index);

/**
 * Prepends @c value to @c list.
 */
void VuoListPrependValue_VuoImageFormat(VuoList_VuoImageFormat list, const VuoImageFormat value);

/**
 * Appends @c value to @c list.
 */
void VuoListAppendValue_VuoImageFormat(VuoList_VuoImageFormat list, const VuoImageFormat value);

/**
 * Swaps the value at `indexA` with the value at `indexB`.
 */
void VuoListExchangeValues_VuoImageFormat(VuoList_VuoImageFormat list, const unsigned long indexA, const unsigned long indexB);

/**
 * Generates a random permutation of `list`.
 *
 * `chaos` ranges from 0 to 1.  When `chaos` is 1, a full Fisher–Yates shuffle is performed.  When less than 1, fewer iterations are performed.
 */
void VuoListShuffle_VuoImageFormat(VuoList_VuoImageFormat list, const double chaos);

/**
 * Reverses the order of the items in `list`.
 */
void VuoListReverse_VuoImageFormat(VuoList_VuoImageFormat list);

/**
 * Removes items from the list except those in the range specified by `startIndex` and `itemCount`.
 */
void VuoListCut_VuoImageFormat(VuoList_VuoImageFormat list, const signed long startIndex, const unsigned long itemCount);

/**
 * Removes the first value from @c list.
 */
void VuoListRemoveFirstValue_VuoImageFormat(VuoList_VuoImageFormat list);

/**
 * Removes the last value from @c list.
 */
void VuoListRemoveLastValue_VuoImageFormat(VuoList_VuoImageFormat list);

/**
 * Removes all values from @c list.
 */
void VuoListRemoveAll_VuoImageFormat(VuoList_VuoImageFormat list);

/**
 * Removes the `VuoImageFormat` at `index`.
 * Index values start at 1.
 * Attempting to remove index 0 or an index beyond the last value in the list has no effect.
 */
void VuoListRemoveValue_VuoImageFormat(VuoList_VuoImageFormat list, const unsigned long index);

/**
 * Returns the number of elements in @c list.
 */
unsigned long VuoListGetCount_VuoImageFormat(const VuoList_VuoImageFormat list);

/**
 * Decodes the JSON object @c js to create a new value.
 *
 * @eg{
 *	["uno", "dos", "tres", "catorce"]
 * }
 */
VuoList_VuoImageFormat VuoList_VuoImageFormat_makeFromJson(struct json_object * js);

/**
 * Encodes @c value as a JSON object.
 */
struct json_object * VuoList_VuoImageFormat_getJson(const VuoList_VuoImageFormat value);

/**
 * Produces a brief human-readable summary of @c value.
 */
char * VuoList_VuoImageFormat_getSummary(const VuoList_VuoImageFormat value);

/// @{
/**
 * Automatically generated function.
 */
VuoList_VuoImageFormat VuoList_VuoImageFormat_makeFromString(const char *str);
char * VuoList_VuoImageFormat_getString(const VuoList_VuoImageFormat value);
/// @}

/**
 * @}
 */

#endif
