/**
 * @file
 * VuoList_VuoShader C type definition.
 *
 * @copyright Copyright © 2012–2015 Kosada Incorporated.
 * This code may be modified and distributed under the terms of the MIT License.
 * For more information, see http://vuo.org/license.
 */

#ifndef VuoList_VuoShader_H
#define VuoList_VuoShader_H

/**
 * @ingroup VuoTypes
 * @defgroup VuoList_VuoShader VuoList_VuoShader
 * A list of @ref VuoShader elements.
 *
 * @{
 */

// This header is generated by vuo/type/list/generateVariants.sh.

#if defined(DOXYGEN) || !defined(VuoList_VuoShader_TYPE_DEFINED)
/// @{
/**
 * A list of @ref VuoShader elements.
 */
typedef const struct VuoList_VuoShader_struct { void *l; } * VuoList_VuoShader;
/// @}
#endif

/**
 * Creates a new list of @ref VuoShader elements.
 */
VuoList_VuoShader VuoListCreate_VuoShader(void);

/**
 * Makes a shallow copy of `list` — its items are retained (not copied) by the new list.
 */
VuoList_VuoShader VuoListCopy_VuoShader(const VuoList_VuoShader list);

/**
 * Returns the @ref VuoShader at @c index.
 * Index values start at 1.
 * If the list has no items, returns a default value.
 * Attempting to access an out-of-bounds index returns the first item in the list (if the index is 0), or last item in the list (if the index is greater than the list size).
 */
VuoShader VuoListGetValue_VuoShader(const VuoList_VuoShader list, const unsigned long index);

/**
 * Changes the @ref VuoShader at @c index.
 * Index values start at 1.
 * If the list has no items, nothing is changed.
 * Attempting to change an out-of-bounds index changes the first item in the list (if the index is 0), or last item in the list (if the index is greater than the list size).
 */
void VuoListSetValue_VuoShader(const VuoList_VuoShader list, const VuoShader value, const unsigned long index);

/**
 * Inserts the `VuoShader` immediately before `index`.
 * Index values start at 1.
 * Inserting at index 0 prepends the value to the list.
 * Inserting at an index beyond the last value in the list appends the value to the list.
 */
void VuoListInsertValue_VuoShader(const VuoList_VuoShader list, const VuoShader value, const unsigned long index);

/**
 * Prepends @c value to @c list.
 */
void VuoListPrependValue_VuoShader(VuoList_VuoShader list, const VuoShader value);

/**
 * Appends @c value to @c list.
 */
void VuoListAppendValue_VuoShader(VuoList_VuoShader list, const VuoShader value);

/**
 * Swaps the value at `indexA` with the value at `indexB`.
 */
void VuoListExchangeValues_VuoShader(VuoList_VuoShader list, const unsigned long indexA, const unsigned long indexB);

/**
 * Generates a random permutation of `list`.
 *
 * `chaos` ranges from 0 to 1.  When `chaos` is 1, a full Fisher–Yates shuffle is performed.  When less than 1, fewer iterations are performed.
 */
void VuoListShuffle_VuoShader(VuoList_VuoShader list, const double chaos);

/**
 * Reverses the order of the items in `list`.
 */
void VuoListReverse_VuoShader(VuoList_VuoShader list);

/**
 * Removes items from the list except those in the range specified by `startIndex` and `itemCount`.
 */
void VuoListCut_VuoShader(VuoList_VuoShader list, const signed long startIndex, const unsigned long itemCount);

/**
 * Removes the first value from @c list.
 */
void VuoListRemoveFirstValue_VuoShader(VuoList_VuoShader list);

/**
 * Removes the last value from @c list.
 */
void VuoListRemoveLastValue_VuoShader(VuoList_VuoShader list);

/**
 * Removes all values from @c list.
 */
void VuoListRemoveAll_VuoShader(VuoList_VuoShader list);

/**
 * Removes the `VuoShader` at `index`.
 * Index values start at 1.
 * Attempting to remove index 0 or an index beyond the last value in the list has no effect.
 */
void VuoListRemoveValue_VuoShader(VuoList_VuoShader list, const unsigned long index);

/**
 * Returns the number of elements in @c list.
 */
unsigned long VuoListGetCount_VuoShader(const VuoList_VuoShader list);

/**
 * Decodes the JSON object @c js to create a new value.
 *
 * @eg{
 *	["uno", "dos", "tres", "catorce"]
 * }
 */
VuoList_VuoShader VuoList_VuoShader_makeFromJson(struct json_object * js);

/**
 * Encodes @c value as a JSON object.
 */
struct json_object * VuoList_VuoShader_getJson(const VuoList_VuoShader value);

/**
 * Produces a brief human-readable summary of @c value.
 */
char * VuoList_VuoShader_getSummary(const VuoList_VuoShader value);

/// @{
/**
 * Automatically generated function.
 */
VuoList_VuoShader VuoList_VuoShader_makeFromString(const char *str);
char * VuoList_VuoShader_getString(const VuoList_VuoShader value);
/// @}

/**
 * @}
 */

#endif
